package javax.swing;

import javax.accessibility.AccessibleTable;
import javax.accessibility.AccessibleRelationSet;
import javax.accessibility.AccessibleIcon;
import javax.accessibility.AccessibleValue;
import javax.accessibility.AccessibleEditableText;
import javax.accessibility.AccessibleText;
import javax.accessibility.AccessibleComponent;
import java.awt.IllegalComponentStateException;
import java.util.Locale;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleRole;
import javax.swing.plaf.basic.ComboPopup;
import javax.accessibility.AccessibleSelection;
import javax.accessibility.AccessibleAction;
import java.io.Serializable;
import java.beans.PropertyChangeEvent;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.event.KeyEvent;
import javax.swing.event.ListDataEvent;
import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.EventQueue;
import java.awt.event.ItemEvent;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import java.awt.event.ItemListener;
import java.beans.Transient;
import java.awt.Component;
import javax.swing.plaf.ComboBoxUI;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import java.util.Vector;
import java.beans.PropertyChangeListener;
import javax.accessibility.Accessible;
import java.awt.event.ActionListener;
import javax.swing.event.ListDataListener;
import java.awt.ItemSelectable;

public class JComboBox<E> extends JComponent implements ItemSelectable, ListDataListener, ActionListener, Accessible
{
    private static final String uiClassID = "ComboBoxUI";
    protected ComboBoxModel<E> dataModel;
    protected ListCellRenderer<? super E> renderer;
    protected ComboBoxEditor editor;
    protected int maximumRowCount;
    protected boolean isEditable;
    protected JComboBox.KeySelectionManager keySelectionManager;
    protected String actionCommand;
    protected boolean lightWeightPopupEnabled;
    protected Object selectedItemReminder;
    private E prototypeDisplayValue;
    private boolean firingActionEvent;
    private boolean selectingItem;
    private Action action;
    private PropertyChangeListener actionPropertyChangeListener;
    
    public JComboBox(ComboBoxModel<E> model) {
        this.maximumRowCount = 8;
        this.isEditable = false;
        this.keySelectionManager = null;
        this.actionCommand = "comboBoxChanged";
        this.lightWeightPopupEnabled = JPopupMenu.getDefaultLightWeightPopupEnabled();
        this.selectedItemReminder = null;
        this.firingActionEvent = false;
        this.selectingItem = false;
        this.setModel(model);
        this.init();
    }
    
    public JComboBox(E[] array) {
        this.maximumRowCount = 8;
        this.isEditable = false;
        this.keySelectionManager = null;
        this.actionCommand = "comboBoxChanged";
        this.lightWeightPopupEnabled = JPopupMenu.getDefaultLightWeightPopupEnabled();
        this.selectedItemReminder = null;
        this.firingActionEvent = false;
        this.selectingItem = false;
        this.setModel(new DefaultComboBoxModel(array));
        this.init();
    }
    
    public JComboBox(Vector<E> vector) {
        this.maximumRowCount = 8;
        this.isEditable = false;
        this.keySelectionManager = null;
        this.actionCommand = "comboBoxChanged";
        this.lightWeightPopupEnabled = JPopupMenu.getDefaultLightWeightPopupEnabled();
        this.selectedItemReminder = null;
        this.firingActionEvent = false;
        this.selectingItem = false;
        this.setModel(new DefaultComboBoxModel(vector));
        this.init();
    }
    
    public JComboBox() {
        this.maximumRowCount = 8;
        this.isEditable = false;
        this.keySelectionManager = null;
        this.actionCommand = "comboBoxChanged";
        this.lightWeightPopupEnabled = JPopupMenu.getDefaultLightWeightPopupEnabled();
        this.selectedItemReminder = null;
        this.firingActionEvent = false;
        this.selectingItem = false;
        this.setModel(new DefaultComboBoxModel());
        this.init();
    }
    
    private void init() {
        this.installAncestorListener();
        this.setUIProperty("opaque", Boolean.valueOf(true));
        this.updateUI();
    }
    
    protected void installAncestorListener() {
        this.addAncestorListener(new AncestorListener() {
            @Override
            public void ancestorAdded(AncestorEvent ancestorEvent) {
                JComboBox.this.hidePopup();
            }
            
            @Override
            public void ancestorRemoved(AncestorEvent ancestorEvent) {
                JComboBox.this.hidePopup();
            }
            
            @Override
            public void ancestorMoved(AncestorEvent ancestorEvent) {
                if (ancestorEvent.getSource() != JComboBox.this) {
                    JComboBox.this.hidePopup();
                }
            }
        });
    }
    
    public void setUI(ComboBoxUI ui) {
        super.setUI(ui);
    }
    
    @Override
    public void updateUI() {
        this.setUI((ComboBoxUI)UIManager.getUI(this));
        ListCellRenderer<Object> renderer = this.getRenderer();
        if ((renderer instanceof Component)) {
            SwingUtilities.updateComponentTreeUI((Component)renderer);
        }
    }
    
    @Override
    public String getUIClassID() {
        return "ComboBoxUI";
    }
    
    public ComboBoxUI getUI() {
        return (ComboBoxUI)this.ui;
    }
    
    public void setModel(ComboBoxModel<E> dataModel) {
        ComboBoxModel<Object> dataModel2 = this.dataModel;
        if (dataModel2 != null) {
            dataModel2.removeListDataListener(this);
        }
        (this.dataModel = dataModel).addListDataListener(this);
        this.selectedItemReminder = this.dataModel.getSelectedItem();
        this.firePropertyChange("model", dataModel2, this.dataModel);
    }
    
    public ComboBoxModel<E> getModel() {
        return this.dataModel;
    }
    
    public void setLightWeightPopupEnabled(boolean lightWeightPopupEnabled) {
        this.firePropertyChange("lightWeightPopupEnabled", this.lightWeightPopupEnabled, this.lightWeightPopupEnabled = lightWeightPopupEnabled);
    }
    
    public boolean isLightWeightPopupEnabled() {
        return this.lightWeightPopupEnabled;
    }
    
    public void setEditable(boolean isEditable) {
        this.firePropertyChange("editable", this.isEditable, this.isEditable = isEditable);
    }
    
    public boolean isEditable() {
        return this.isEditable;
    }
    
    public void setMaximumRowCount(int maximumRowCount) {
        this.firePropertyChange("maximumRowCount", this.maximumRowCount, this.maximumRowCount = maximumRowCount);
    }
    
    public int getMaximumRowCount() {
        return this.maximumRowCount;
    }
    
    public void setRenderer(ListCellRenderer<? super E> renderer) {
        this.firePropertyChange("renderer", this.renderer, this.renderer = renderer);
        this.invalidate();
    }
    
    public ListCellRenderer<? super E> getRenderer() {
        return this.renderer;
    }
    
    public void setEditor(ComboBoxEditor editor) {
        ComboBoxEditor editor2 = this.editor;
        if (this.editor != null) {
            this.editor.removeActionListener(this);
        }
        this.editor = editor;
        if (this.editor != null) {
            this.editor.addActionListener(this);
        }
        this.firePropertyChange("editor", editor2, this.editor);
    }
    
    public ComboBoxEditor getEditor() {
        return this.editor;
    }
    
    public void setSelectedItem(Object o) {
        Object selectedItemReminder = this.selectedItemReminder;
        if (selectedItemReminder == null || !selectedItemReminder.equals(o)) {
            if (o != null && !this.isEditable()) {
                boolean n = false;
                int i = 0;
                while (i < this.dataModel.getSize()) {
                    Object element = this.dataModel.getElementAt(i);
                    if (o.equals(element)) {
                        n = true;
                        o = element;
                        break;
                    }
                    ++i;
                }
                if (n == 0) {
                    return;
                }
            }
            this.selectingItem = true;
            this.dataModel.setSelectedItem(o);
            this.selectingItem = false;
            if (this.selectedItemReminder != this.dataModel.getSelectedItem()) {
                this.selectedItemChanged();
            }
        }
        this.fireActionEvent();
    }
    
    public Object getSelectedItem() {
        return this.dataModel.getSelectedItem();
    }
    
    public void setSelectedIndex(int i) {
        int size = this.dataModel.getSize();
        if (i == -1) {
            this.setSelectedItem(null);
        }
        else {
            if (i < -1 || i >= size) {
                throw new IllegalArgumentException("setSelectedIndex: " + i + " out of bounds");
            }
            this.setSelectedItem(this.dataModel.getElementAt(i));
        }
    }
    
    @Transient
    public int getSelectedIndex() {
        Object selectedItem = this.dataModel.getSelectedItem();
        int i = 0;
        while (i < this.dataModel.getSize()) {
            Object element = this.dataModel.getElementAt(i);
            if (element != null && element.equals(selectedItem)) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    public E getPrototypeDisplayValue() {
        return this.prototypeDisplayValue;
    }
    
    public void setPrototypeDisplayValue(E prototypeDisplayValue) {
        this.firePropertyChange("prototypeDisplayValue", this.prototypeDisplayValue, this.prototypeDisplayValue = prototypeDisplayValue);
    }
    
    public void addItem(E e) {
        this.checkMutableComboBoxModel();
        ((MutableComboBoxModel)this.dataModel).addElement(e);
    }
    
    public void insertItemAt(E e, int n) {
        this.checkMutableComboBoxModel();
        ((MutableComboBoxModel)this.dataModel).insertElementAt(e, n);
    }
    
    public void removeItem(Object o) {
        this.checkMutableComboBoxModel();
        ((MutableComboBoxModel)this.dataModel).removeElement(o);
    }
    
    public void removeItemAt(int n) {
        this.checkMutableComboBoxModel();
        ((MutableComboBoxModel)this.dataModel).removeElementAt(n);
    }
    
    public void removeAllItems() {
        this.checkMutableComboBoxModel();
        MutableComboBoxModel<Object> mutableComboBoxModel = (MutableComboBoxModel)this.dataModel;
        int size = mutableComboBoxModel.getSize();
        if ((mutableComboBoxModel instanceof DefaultComboBoxModel)) {
            ((DefaultComboBoxModel)mutableComboBoxModel).removeAllElements();
        }
        else {
            for (int i = 0; i < size; ++i) {
                mutableComboBoxModel.removeElement(mutableComboBoxModel.getElementAt(0));
            }
        }
        this.selectedItemReminder = null;
        if (this.isEditable()) {
            this.editor.setItem(null);
        }
    }
    
    void checkMutableComboBoxModel() {
        if (!(this.dataModel instanceof MutableComboBoxModel)) {
            throw new RuntimeException("Cannot use this method with a non-Mutable data model.");
        }
        return;
    }
    
    public void showPopup() {
        this.setPopupVisible(true);
    }
    
    public void hidePopup() {
        this.setPopupVisible(false);
    }
    
    public void setPopupVisible(boolean b) {
        this.getUI().setPopupVisible(this, b);
    }
    
    public boolean isPopupVisible() {
        return this.getUI().isPopupVisible(this);
    }
    
    @Override
    public void addItemListener(ItemListener l) {
        this.listenerList.add(ItemListener.class, l);
    }
    
    @Override
    public void removeItemListener(ItemListener l) {
        this.listenerList.remove(ItemListener.class, l);
    }
    
    public ItemListener[] getItemListeners() {
        return (ItemListener[])this.listenerList.getListeners(ItemListener.class);
    }
    
    public void addActionListener(ActionListener l) {
        this.listenerList.add(ActionListener.class, l);
    }
    
    public void removeActionListener(ActionListener l) {
        if (l != null && this.getAction() == l) {
            this.setAction(null);
        }
        else {
            this.listenerList.remove(ActionListener.class, l);
        }
    }
    
    public ActionListener[] getActionListeners() {
        return (ActionListener[])this.listenerList.getListeners(ActionListener.class);
    }
    
    public void addPopupMenuListener(PopupMenuListener l) {
        this.listenerList.add(PopupMenuListener.class, l);
    }
    
    public void removePopupMenuListener(PopupMenuListener l) {
        this.listenerList.remove(PopupMenuListener.class, l);
    }
    
    public PopupMenuListener[] getPopupMenuListeners() {
        return (PopupMenuListener[])this.listenerList.getListeners(PopupMenuListener.class);
    }
    
    public void firePopupMenuWillBecomeVisible() {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == PopupMenuListener.class) {
                if (o == null) {
                    o = new PopupMenuEvent(this);
                }
                ((PopupMenuListener)listenerList[i + 1]).popupMenuWillBecomeVisible(o);
            }
        }
    }
    
    public void firePopupMenuWillBecomeInvisible() {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == PopupMenuListener.class) {
                if (o == null) {
                    o = new PopupMenuEvent(this);
                }
                ((PopupMenuListener)listenerList[i + 1]).popupMenuWillBecomeInvisible(o);
            }
        }
    }
    
    public void firePopupMenuCanceled() {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == PopupMenuListener.class) {
                if (o == null) {
                    o = new PopupMenuEvent(this);
                }
                ((PopupMenuListener)listenerList[i + 1]).popupMenuCanceled(o);
            }
        }
    }
    
    public void setActionCommand(String actionCommand) {
        this.actionCommand = actionCommand;
    }
    
    public String getActionCommand() {
        return this.actionCommand;
    }
    
    public void setAction(Action action) {
        Action action2 = this.getAction();
        if (this.action == null || !this.action.equals(action)) {
            this.action = action;
            if (action2 != null) {
                this.removeActionListener(action2);
                action2.removePropertyChangeListener(this.actionPropertyChangeListener);
                this.actionPropertyChangeListener = null;
            }
            this.configurePropertiesFromAction(this.action);
            if (this.action != null) {
                if (!this.isListener(ActionListener.class, this.action)) {
                    this.addActionListener(this.action);
                }
                this.actionPropertyChangeListener = this.createActionPropertyChangeListener(this.action);
                this.action.addPropertyChangeListener(this.actionPropertyChangeListener);
            }
            this.firePropertyChange("action", action2, this.action);
        }
    }
    
    private boolean isListener(Class clazz, ActionListener actionListener) {
        boolean n = false;
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == clazz && listenerList[i + 1] == actionListener) {
                n = true;
            }
        }
        return n;
    }
    
    public Action getAction() {
        return this.action;
    }
    
    protected void configurePropertiesFromAction(Action actionCommandFromAction) {
        AbstractAction.setEnabledFromAction(this, actionCommandFromAction);
        AbstractAction.setToolTipTextFromAction(this, actionCommandFromAction);
        this.setActionCommandFromAction(actionCommandFromAction);
    }
    
    protected PropertyChangeListener createActionPropertyChangeListener(Action action) {
        return new JComboBox.ComboBoxActionPropertyChangeListener(this, action);
    }
    
    protected void actionPropertyChanged(Action actionCommandFromAction, String s) {
        if (s == "ActionCommandKey") {
            this.setActionCommandFromAction(actionCommandFromAction);
        }
        else if (s == "enabled") {
            AbstractAction.setEnabledFromAction(this, actionCommandFromAction);
        }
        else if ("ShortDescription" == s) {
            AbstractAction.setToolTipTextFromAction(this, actionCommandFromAction);
        }
    }
    
    private void setActionCommandFromAction(Action action) {
        this.setActionCommand((action != null) ? ((String)action.getValue("ActionCommandKey")) : null);
    }
    
    protected void fireItemStateChanged(ItemEvent itemEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ItemListener.class) {
                ((ItemListener)listenerList[i + 1]).itemStateChanged(itemEvent);
            }
        }
    }
    
    protected void fireActionEvent() {
        if (!this.firingActionEvent) {
            this.firingActionEvent = true;
            Object o = null;
            Object[] listenerList = this.listenerList.getListenerList();
            long mostRecentEventTime = EventQueue.getMostRecentEventTime();
            int modifiers = 0;
            AWTEvent currentEvent = EventQueue.getCurrentEvent();
            if ((currentEvent instanceof InputEvent)) {
                modifiers = ((InputEvent)currentEvent).getModifiers();
            }
            else if ((currentEvent instanceof ActionEvent)) {
                modifiers = ((ActionEvent)currentEvent).getModifiers();
            }
            for (int i = listenerList.length - 2; i >= 0; i -= 2) {
                if (listenerList[i] == ActionListener.class) {
                    if (o == null) {
                        o = new ActionEvent(this, 1001, this.getActionCommand(), mostRecentEventTime, modifiers);
                    }
                    ((ActionListener)listenerList[i + 1]).actionPerformed(o);
                }
            }
            this.firingActionEvent = false;
        }
    }
    
    protected void selectedItemChanged() {
        if (this.selectedItemReminder != null) {
            this.fireItemStateChanged(new ItemEvent(this, 701, this.selectedItemReminder, 2));
        }
        this.selectedItemReminder = this.dataModel.getSelectedItem();
        if (this.selectedItemReminder != null) {
            this.fireItemStateChanged(new ItemEvent(this, 701, this.selectedItemReminder, 1));
        }
    }
    
    @Override
    public Object[] getSelectedObjects() {
        Object selectedItem = this.getSelectedItem();
        if (selectedItem == null) {
            return new Object[0];
        }
        return new Object[] { selectedItem };
    }
    
    @Override
    public void actionPerformed(ActionEvent actionEvent) {
        ComboBoxEditor editor = this.getEditor();
        if (editor != null && actionEvent != null && (editor == actionEvent.getSource() || editor.getEditorComponent() == actionEvent.getSource())) {
            this.setPopupVisible(false);
            this.getModel().setSelectedItem(editor.getItem());
            String actionCommand = this.getActionCommand();
            this.setActionCommand("comboBoxEdited");
            this.fireActionEvent();
            this.setActionCommand(actionCommand);
        }
    }
    
    @Override
    public void contentsChanged(ListDataEvent listDataEvent) {
        Object selectedItemReminder = this.selectedItemReminder;
        Object selectedItem = this.dataModel.getSelectedItem();
        if (selectedItemReminder == null || !selectedItemReminder.equals(selectedItem)) {
            this.selectedItemChanged();
            if (!this.selectingItem) {
                this.fireActionEvent();
            }
        }
    }
    
    @Override
    public void intervalAdded(ListDataEvent listDataEvent) {
        if (this.selectedItemReminder != this.dataModel.getSelectedItem()) {
            this.selectedItemChanged();
        }
    }
    
    @Override
    public void intervalRemoved(ListDataEvent listDataEvent) {
        this.contentsChanged(listDataEvent);
    }
    
    public boolean selectWithKeyChar(char c) {
        if (this.keySelectionManager == null) {
            this.keySelectionManager = this.createDefaultKeySelectionManager();
        }
        int selectionForKey = this.keySelectionManager.selectionForKey(c, this.getModel());
        if (selectionForKey != -1) {
            this.setSelectedIndex(selectionForKey);
            return true;
        }
        return false;
    }
    
    @Override
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);
        this.firePropertyChange("enabled", !this.isEnabled(), this.isEnabled());
    }
    
    public void configureEditor(ComboBoxEditor comboBoxEditor, Object item) {
        comboBoxEditor.setItem(item);
    }
    
    public void processKeyEvent(KeyEvent keyEvent) {
        if (keyEvent.getKeyCode() == 9) {
            this.hidePopup();
        }
        super.processKeyEvent(keyEvent);
    }
    
    @Override
    protected boolean processKeyBinding(KeyStroke keyStroke, KeyEvent keyEvent, int n, boolean b) {
        if (super.processKeyBinding(keyStroke, keyEvent, n, b)) {
            return true;
        }
        if (!this.isEditable() || n != 0 || this.getEditor() == null || !Boolean.TRUE.equals(this.getClientProperty("JComboBox.isTableCellEditor"))) {
            return false;
        }
        Component editorComponent = this.getEditor().getEditorComponent();
        return editorComponent instanceof JComponent && ((JComponent)editorComponent).processKeyBinding(keyStroke, keyEvent, 0, b);
    }
    
    public void setKeySelectionManager(JComboBox.KeySelectionManager keySelectionManager) {
        this.keySelectionManager = keySelectionManager;
    }
    
    public JComboBox.KeySelectionManager getKeySelectionManager() {
        return this.keySelectionManager;
    }
    
    public int getItemCount() {
        return this.dataModel.getSize();
    }
    
    public E getItemAt(int n) {
        return this.dataModel.getElementAt(n);
    }
    
    protected JComboBox.KeySelectionManager createDefaultKeySelectionManager() {
        return new JComboBox.DefaultKeySelectionManager();
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("ComboBoxUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",isEditable=" + (this.isEditable ? "true" : "false") + ",lightWeightPopupEnabled=" + (this.lightWeightPopupEnabled ? "true" : "false") + ",maximumRowCount=" + this.maximumRowCount + ",selectedItemReminder=" + ((this.selectedItemReminder != null) ? this.selectedItemReminder.toString() : "");
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JComboBox.AccessibleJComboBox();
        }
        return this.accessibleContext;
    }
    
    private static class ComboBoxActionPropertyChangeListener extends ActionPropertyChangeListener<JComboBox<?>>
    {
        ComboBoxActionPropertyChangeListener(JComboBox<?> comboBox, Action action) {
            super(comboBox, action);
        }
        
        @Override
        protected void actionPropertyChanged(JComboBox<?> comboBox, Action action, PropertyChangeEvent propertyChangeEvent) {
            if (AbstractAction.shouldReconfigure(propertyChangeEvent)) {
                comboBox.configurePropertiesFromAction(action);
            }
            else {
                comboBox.actionPropertyChanged(action, propertyChangeEvent.getPropertyName());
            }
        }
    }
    
    class DefaultKeySelectionManager implements JComboBox.KeySelectionManager, Serializable
    {
        @Override
        public int selectionForKey(char c, ComboBoxModel comboBoxModel) {
            int i = -1;
            Object selectedItem = comboBoxModel.getSelectedItem();
            if (selectedItem != null) {
                int j = 0;
                while (j < comboBoxModel.getSize()) {
                    if (selectedItem == comboBoxModel.getElementAt(j)) {
                        i = j;
                        break;
                    }
                    ++j;
                }
            }
            int char1 = ("" + c).toLowerCase().charAt(0);
            for (i = ++i; i < comboBoxModel.getSize(); ++i) {
                Object element = comboBoxModel.getElementAt(i);
                if (element != null && element.toString() != null) {
                    String lowerCase = element.toString().toLowerCase();
                    if (lowerCase.length() > 0 && lowerCase.charAt(0) == char1) {
                        return i;
                    }
                }
            }
            for (int k = 0; k < i; ++k) {
                Object element2 = comboBoxModel.getElementAt(k);
                if (element2 != null && element2.toString() != null) {
                    String lowerCase2 = element2.toString().toLowerCase();
                    if (lowerCase2.length() > 0 && lowerCase2.charAt(0) == char1) {
                        return k;
                    }
                }
            }
            return -1;
        }
    }
    
    protected class AccessibleJComboBox extends JComponent.AccessibleJComponent implements AccessibleAction, AccessibleSelection
    {
        private JList popupList;
        private Accessible previousSelectedAccessible;
        private JComboBox.AccessibleJComboBox.EditorAccessibleContext editorAccessibleContext;
        
        public AccessibleJComboBox() {
            this.previousSelectedAccessible = null;
            this.editorAccessibleContext = null;
            JComboBox.this.addPropertyChangeListener(new JComboBox.AccessibleJComboBox.AccessibleJComboBoxPropertyChangeListener());
            this.setEditorNameAndDescription();
            Accessible accessibleChild = JComboBox.this.getUI().getAccessibleChild(JComboBox.this, 0);
            if ((accessibleChild instanceof ComboPopup)) {
                (this.popupList = ((ComboPopup)accessibleChild).getList()).addListSelectionListener(new JComboBox.AccessibleJComboBox.AccessibleJComboBoxListSelectionListener());
            }
            JComboBox.this.addPopupMenuListener(new JComboBox.AccessibleJComboBox.AccessibleJComboBoxPopupMenuListener());
        }
        
        private void setEditorNameAndDescription() {
            ComboBoxEditor editor = JComboBox.this.getEditor();
            if (editor != null) {
                Component editorComponent = editor.getEditorComponent();
                if ((editorComponent instanceof Accessible)) {
                    AccessibleContext accessibleContext = editorComponent.getAccessibleContext();
                    if (accessibleContext != null) {
                        accessibleContext.setAccessibleName(this.getAccessibleName());
                        accessibleContext.setAccessibleDescription(this.getAccessibleDescription());
                    }
                }
            }
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            if (JComboBox.this.ui != null) {
                return JComboBox.this.ui.getAccessibleChildrenCount(JComboBox.this);
            }
            return super.getAccessibleChildrenCount();
        }
        
        @Override
        public Accessible getAccessibleChild(int i) {
            if (JComboBox.this.ui != null) {
                return JComboBox.this.ui.getAccessibleChild(JComboBox.this, i);
            }
            return super.getAccessibleChild(i);
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.COMBO_BOX;
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            if (accessibleStateSet == null) {
                accessibleStateSet = new AccessibleStateSet();
            }
            if (JComboBox.this.isPopupVisible()) {
                accessibleStateSet.add(AccessibleState.EXPANDED);
                return accessibleStateSet;
            }
            accessibleStateSet.add(AccessibleState.COLLAPSED);
            return accessibleStateSet;
        }
        
        @Override
        public AccessibleAction getAccessibleAction() {
            return this;
        }
        
        @Override
        public String getAccessibleActionDescription(int n) {
            if (n == 0) {
                return UIManager.getString("ComboBox.togglePopupText");
            }
            return null;
        }
        
        @Override
        public int getAccessibleActionCount() {
            return 1;
        }
        
        @Override
        public boolean doAccessibleAction(int n) {
            if (n == 0) {
                JComboBox.this.setPopupVisible(!JComboBox.this.isPopupVisible());
                return true;
            }
            return false;
        }
        
        @Override
        public AccessibleSelection getAccessibleSelection() {
            return this;
        }
        
        @Override
        public int getAccessibleSelectionCount() {
            if (JComboBox.this.getSelectedItem() != null) {
                return 1;
            }
            return 0;
        }
        
        @Override
        public Accessible getAccessibleSelection(int n) {
            Accessible accessibleChild = JComboBox.this.getUI().getAccessibleChild(JComboBox.this, 0);
            if (accessibleChild != null && accessibleChild instanceof ComboPopup) {
                AccessibleContext accessibleContext = ((ComboPopup)accessibleChild).getList().getAccessibleContext();
                if (accessibleContext != null) {
                    AccessibleSelection accessibleSelection = accessibleContext.getAccessibleSelection();
                    if (accessibleSelection != null) {
                        return accessibleSelection.getAccessibleSelection(n);
                    }
                }
            }
            return null;
        }
        
        @Override
        public boolean isAccessibleChildSelected(int n) {
            return JComboBox.this.getSelectedIndex() == n;
        }
        
        @Override
        public void addAccessibleSelection(int selectedIndex) {
            this.clearAccessibleSelection();
            JComboBox.this.setSelectedIndex(selectedIndex);
        }
        
        @Override
        public void removeAccessibleSelection(int n) {
            if (JComboBox.this.getSelectedIndex() == n) {
                this.clearAccessibleSelection();
            }
        }
        
        @Override
        public void clearAccessibleSelection() {
            JComboBox.this.setSelectedIndex(-1);
        }
        
        @Override
        public void selectAllAccessibleSelection() {
        }
        
        static void access$300(JComboBox.AccessibleJComboBox accessibleJComboBox) {
            accessibleJComboBox.setEditorNameAndDescription();
        }
        
        private class AccessibleJComboBoxPropertyChangeListener implements PropertyChangeListener
        {
            @Override
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                if (propertyChangeEvent.getPropertyName() == "editor") {
                    JComboBox.AccessibleJComboBox.access$300(JComboBox.AccessibleJComboBox.this);
                }
            }
        }
        
        private class AccessibleJComboBoxPopupMenuListener implements PopupMenuListener
        {
            @Override
            public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
                if (JComboBox.AccessibleJComboBox.this.popupList == null) {
                    return;
                }
                int selectedIndex = JComboBox.AccessibleJComboBox.this.popupList.getSelectedIndex();
                if (selectedIndex < 0) {
                    return;
                }
                JComboBox.AccessibleJComboBox.this.previousSelectedAccessible = JComboBox.AccessibleJComboBox.this.popupList.getAccessibleContext().getAccessibleChild(selectedIndex);
                return;
            }
            
            @Override
            public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
            }
            
            @Override
            public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
            }
        }
        
        private class AccessibleJComboBoxListSelectionListener implements ListSelectionListener
        {
            @Override
            public void valueChanged(ListSelectionEvent listSelectionEvent) {
                if (JComboBox.AccessibleJComboBox.this.popupList == null) {
                    return;
                }
                int selectedIndex = JComboBox.AccessibleJComboBox.this.popupList.getSelectedIndex();
                if (selectedIndex < 0) {
                    return;
                }
                Accessible accessibleChild = JComboBox.AccessibleJComboBox.this.popupList.getAccessibleContext().getAccessibleChild(selectedIndex);
                if (accessibleChild == null) {
                    return;
                }
                if (JComboBox.AccessibleJComboBox.this.previousSelectedAccessible != null) {
                    JComboBox.AccessibleJComboBox.this.firePropertyChange("AccessibleState", null, new PropertyChangeEvent(JComboBox.AccessibleJComboBox.this.previousSelectedAccessible, "AccessibleState", AccessibleState.FOCUSED, null));
                }
                JComboBox.AccessibleJComboBox.this.firePropertyChange("AccessibleState", null, new PropertyChangeEvent(accessibleChild, "AccessibleState", null, AccessibleState.FOCUSED));
                JComboBox.AccessibleJComboBox.this.firePropertyChange("AccessibleActiveDescendant", JComboBox.AccessibleJComboBox.this.previousSelectedAccessible, accessibleChild);
                JComboBox.AccessibleJComboBox.this.previousSelectedAccessible = accessibleChild;
                return;
            }
        }
        
        private class AccessibleEditor implements Accessible
        {
            @Override
            public AccessibleContext getAccessibleContext() {
                if (JComboBox.AccessibleJComboBox.this.editorAccessibleContext == null) {
                    Component editorComponent = JComboBox.this.getEditor().getEditorComponent();
                    if ((editorComponent instanceof Accessible)) {
                        JComboBox.AccessibleJComboBox.this.editorAccessibleContext = new JComboBox.AccessibleJComboBox.EditorAccessibleContext((Accessible)editorComponent);
                    }
                }
                return JComboBox.AccessibleJComboBox.this.editorAccessibleContext;
            }
        }
        
        private class EditorAccessibleContext extends AccessibleContext
        {
            private AccessibleContext ac;
            
            private EditorAccessibleContext() {
            }
            
            EditorAccessibleContext(Accessible accessible) {
                this.ac = accessible.getAccessibleContext();
            }
            
            @Override
            public String getAccessibleName() {
                return this.ac.getAccessibleName();
            }
            
            @Override
            public void setAccessibleName(String accessibleName) {
                this.ac.setAccessibleName(accessibleName);
            }
            
            @Override
            public String getAccessibleDescription() {
                return this.ac.getAccessibleDescription();
            }
            
            @Override
            public void setAccessibleDescription(String accessibleDescription) {
                this.ac.setAccessibleDescription(accessibleDescription);
            }
            
            @Override
            public AccessibleRole getAccessibleRole() {
                return this.ac.getAccessibleRole();
            }
            
            @Override
            public AccessibleStateSet getAccessibleStateSet() {
                return this.ac.getAccessibleStateSet();
            }
            
            @Override
            public Accessible getAccessibleParent() {
                return this.ac.getAccessibleParent();
            }
            
            @Override
            public void setAccessibleParent(Accessible accessibleParent) {
                this.ac.setAccessibleParent(accessibleParent);
            }
            
            @Override
            public int getAccessibleIndexInParent() {
                return JComboBox.this.getSelectedIndex();
            }
            
            @Override
            public int getAccessibleChildrenCount() {
                return this.ac.getAccessibleChildrenCount();
            }
            
            @Override
            public Accessible getAccessibleChild(int n) {
                return this.ac.getAccessibleChild(n);
            }
            
            @Override
            public Locale getLocale() throws IllegalComponentStateException {
                return this.ac.getLocale();
            }
            
            @Override
            public void addPropertyChangeListener(PropertyChangeListener listener) {
                this.ac.addPropertyChangeListener(listener);
            }
            
            @Override
            public void removePropertyChangeListener(PropertyChangeListener listener) {
                this.ac.removePropertyChangeListener(listener);
            }
            
            @Override
            public AccessibleAction getAccessibleAction() {
                return this.ac.getAccessibleAction();
            }
            
            @Override
            public AccessibleComponent getAccessibleComponent() {
                return this.ac.getAccessibleComponent();
            }
            
            @Override
            public AccessibleSelection getAccessibleSelection() {
                return this.ac.getAccessibleSelection();
            }
            
            @Override
            public AccessibleText getAccessibleText() {
                return this.ac.getAccessibleText();
            }
            
            @Override
            public AccessibleEditableText getAccessibleEditableText() {
                return this.ac.getAccessibleEditableText();
            }
            
            @Override
            public AccessibleValue getAccessibleValue() {
                return this.ac.getAccessibleValue();
            }
            
            @Override
            public AccessibleIcon[] getAccessibleIcon() {
                return this.ac.getAccessibleIcon();
            }
            
            @Override
            public AccessibleRelationSet getAccessibleRelationSet() {
                return this.ac.getAccessibleRelationSet();
            }
            
            @Override
            public AccessibleTable getAccessibleTable() {
                return this.ac.getAccessibleTable();
            }
            
            @Override
            public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
                this.ac.firePropertyChange(propertyName, oldValue, newValue);
            }
        }
    }
    
    public interface KeySelectionManager
    {
        public abstract int selectionForKey(char p0, ComboBoxModel p1);
    }
}
